{% extends "base_generic.html" %}

{% block title %}{{ title }}{% endblock %}

{% block heading %}{{ title }}{% endblock %}

{% block content %}

    <div class="row">
        <div class="large-12 columns">
            <h3>Customize collection {{coll_id}}</h3>
        </div>
    </div>

    <div class="row">
        <div class="large-12 columns">
            <form method="POST" action="#">
                {% csrf_token %}

                <div class="row">
                    <div class="medium-4 columns">
                        <h4>Record types</h4>
                        <select name="typelist" size="{{ select_rows }}" 
                                class="small-size-4 medium-size-12 medium-add-margin">
                            {% for t in types %}
                            <option value="{{t.get_id}}">{{t.get_label}}</option>
                            {% endfor %}
                         </select>
                        <input type="submit" name="type_new"    value="New" />
                        <input type="submit" name="type_copy"   value="Copy" />
                        <input type="submit" name="type_edit"   value="Edit" />
                        <input type="submit" name="type_delete" value="Delete" />
                    </div>
                    <div class="medium-4 columns">
                        <h4>List views</h4>
                        <select name="listlist" size="{{ select_rows }}" 
                                class="small-size-4 medium-size-12 medium-add-margin">
                            {% for l in lists %}
                            <option value="{{l.get_id}}">{{l.get_label}}</option>
                            {% endfor %}
                         </select>
                        <input type="submit" name="list_new"    value="New" />
                        <input type="submit" name="list_copy"   value="Copy" />
                        <input type="submit" name="list_edit"   value="Edit" />
                        <input type="submit" name="list_delete" value="Delete" />
                    </div>
                    <div class="medium-4 columns">
                        <h4>Record views</h4>
                        <select name="viewlist" size="{{ select_rows }}" 
                                class="small-size-4 medium-size-12 medium-add-margin">
                            {% for v in views %}
                            <option value="{{v.get_id}}">{{v.get_label}}</option>
                            {% endfor %}
                         </select>
                        <input type="submit" name="view_new"    value="New" />
                        <input type="submit" name="view_copy"   value="Copy" />
                        <input type="submit" name="view_edit"   value="Edit" />
                        <input type="submit" name="view_delete" value="Delete" />
                    </div>
                </div>

                <div class="row">
                    <div class="medium-4 columns">
                        <input type="submit" name="close" value="Close" />
                    </div>
                </div>

                <input type="hidden" name="continuation_url" value="{{ continuation_url }}" />
            </form>
        </div>
    </div>
{% endblock %}
