<!-- field/annalist_edit_view_choice.html -->
<div class="row">
  <div class="small-9 columns">
    <select name="{{repeat_prefix}}{{field.field_name}}">
      {% for opt in field.options %}
        {% if opt.value == field.field_value %}
        <option value="{{opt.value}}" selected="selected">{{opt.label}}</option>
        {% else %}
        <option value="{{opt.value}}">{{opt.label}}</option>
        {% endif %}
      {% endfor %}
    </select>
  </div>
  <div class="small-3 columns">
    <input type="submit" name="use_view" value="Show view" />
  </div>
</div>
